@@ -5,8 +5,8 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2023
-;; Version: 2.2
-;; Package-Requires: ((emacs "28.1") (compat "30"))
+;; Version: 2.3
+;; Package-Requires: ((emacs "29.1") (compat "30"))
 ;; URL: https://github.com/minad/jinx
 ;; Keywords: convenience, text
 
@@ -154,6 +154,13 @@ These faces mark regions which should be excluded in spell
 checking."
   :type '(alist :key-type symbol :value-type (choice symbol (repeat face))))
 
+(defcustom jinx-exclude-properties
+  '((t read-only))
+  "Alist of properties per major mode.
+These properties mark regions which should be excluded in spell
+checking."
+  :type '(alist :key-type symbol :value-type (choice symbol (repeat symbol))))
+
 (defcustom jinx-exclude-regexps
   '((emacs-lisp-mode "Package-Requires:.*$")
     (t "[A-Z]+\\>"         ;; Uppercase words
@@ -163,7 +170,10 @@ checking."
        "<?[-+_.~a-zA-Z][-+_.~:a-zA-Z0-9]*@[-.a-zA-Z0-9]+>?" ;; Email
        "\\(?:Local Variables\\|End\\):\\s-*$" ;; Local variable indicator
        "jinx-\\(?:languages\\|local-words\\):\\s-+.*$")) ;; Local variables
-  "List of excluded regexps per major mode."
+  "List of excluded regexps per major mode.
+The regexp must match at the beginning of the spell-checked word.
+Therefore `jinx-exclude-regexps' cannot be used to exclude larger parts
+of a buffer.  Write a custom predicate instead, see `jinx--predicates'."
   :type '(alist :key-type symbol :value-type (choice symbol (repeat regexp))))
 
 (defcustom jinx-suggestion-distance 3
@@ -278,8 +288,9 @@ checking."
   "Hooks which reschedule the spell checking timer, see `jinx--reschedule'.")
 
 (defvar jinx--predicates
-  (list #'jinx--face-ignored-p
-        #'jinx--regexp-ignored-p
+  (list #'jinx--face-excluded-p
+        #'jinx--regexp-excluded-p
+        #'jinx--property-excluded-p
         #'jinx--word-valid-p)
   "Predicate functions called at point with argument START.
 Predicate should return t if the word before point is valid.
@@ -319,6 +330,9 @@ Predicate may return a position to skip forward.")
 (defvar-local jinx--exclude-faces nil
   "List of excluded faces.")
 
+(defvar-local jinx--exclude-properties nil
+  "List of excluded properties.")
+
 (defvar-local jinx--include-faces nil
   "List of included faces.")
 
@@ -361,18 +375,18 @@ Predicate may return a position to skip forward.")
 
 ;;;; Predicates
 
-(defun jinx--regexp-ignored-p (start)
-  "Return non-nil if word at START matches ignore regexps."
+(defun jinx--regexp-excluded-p (pos)
+  "Return non-nil if word at POS matches a regexp from `jinx-exclude-regexps'."
   (save-excursion
-    (goto-char start)
+    (goto-char pos)
     (when (and jinx--exclude-regexp (looking-at-p jinx--exclude-regexp))
       (save-match-data
         (looking-at jinx--exclude-regexp)
         (match-end 0)))))
 
-(defun jinx--face-ignored-p (start)
-  "Return non-nil if face at START of word is ignored."
-  (let ((face (get-text-property start 'face)))
+(defun jinx--face-excluded-p (pos)
+  "Return non-nil if face at POS is excluded via `jinx-exclude-faces'."
+  (let ((face (get-text-property pos 'face)))
     (or
      (and jinx--include-faces
           (if (listp face)
@@ -383,9 +397,18 @@ Predicate may return a position to skip forward.")
               (cl-loop for f in face thereis (memq f jinx--exclude-faces))
             (memq face jinx--exclude-faces))))))
 
+(defun jinx--property-excluded-p (pos)
+  "Return non-nil if property at POS is excluded via `jinx-exclude-properties'."
+  (cl-loop for prop in jinx--exclude-properties
+           thereis (get-char-property pos prop)))
+
+(defun jinx--read-only-p (pos)
+  "Return non-nil if POS is read-only."
+  (get-text-property pos 'read-only))
+
 (defun jinx--word-valid-p (word)
   "Return non-nil if WORD is valid.
-WORD can also be a start position."
+WORD can also be a position."
   (unless (stringp word)
     (setq word (buffer-substring-no-properties word (point))))
   (or (member word jinx--session-words)
@@ -683,14 +706,8 @@ See `isearch-open-necessary-overlays' and `isearch-open-overlay-temporary'."
 
 (defun jinx--correct-setup ()
   "Setup minibuffer for correction."
-  (use-local-map (make-composed-keymap (list jinx-correct-map) (current-local-map)))
-  ;; TODO Use `eager-display' on Emacs 31
-  (when (and (eq completing-read-function #'completing-read-default)
-             (not (bound-and-true-p vertico-mode))
-             (not (bound-and-true-p icomplete-mode)))
-    (let ((message-log-max nil)
-          (inhibit-message t))
-      (minibuffer-completion-help))))
+  (use-local-map
+   (make-composed-keymap (list jinx-correct-map) (current-local-map))))
 
 (defun jinx--add-suggestion (list ht word group)
   "Add suggestion WORD to LIST and HT.
@@ -797,6 +814,7 @@ Optionally show prompt INFO and insert INITIAL input."
                        (jinx--table-with-metadata
                         (jinx--correct-suggestions word)
                         `((category . jinx)
+                          (eager-display . t)
                           (display-sort-function . ,#'identity)
                           (cycle-sort-function . ,#'identity)
                           (group-function . ,#'jinx--group)
@@ -1115,6 +1133,7 @@ This command dispatches to the following commands:
                        regexps "\\|"))
           jinx--include-faces (jinx--mode-list jinx-include-faces)
           jinx--exclude-faces (jinx--mode-list jinx-exclude-faces)
+          jinx--exclude-properties (jinx--mode-list jinx-exclude-properties)
           jinx--camel (or (eq jinx-camel-modes t)
                           (seq-some #'derived-mode-p jinx-camel-modes))
           jinx--session-words (nconc (split-string jinx-dir-local-words)
@@ -1133,32 +1152,18 @@ This command dispatches to the following commands:
     (jit-lock-unregister #'jinx--mark-pending)
     (jinx--cleanup))))
 
-;; TODO use `:predicate' on Emacs 29
-(defcustom global-jinx-modes '(text-mode prog-mode conf-mode)
-  "List of modes where Jinx should be enabled.
-The variable can either be t, nil or a list of t, nil, mode
-symbols or elements of the form (not modes)."
-  :type '(repeat sexp))
-
 ;;;###autoload
 (define-globalized-minor-mode global-jinx-mode
   jinx-mode jinx--on
-  :group 'jinx)
+  :group 'jinx
+  :predicate '(text-mode prog-mode conf-mode))
 
 (defun jinx--on ()
   "Turn `jinx-mode' on."
-  (when (and (not (or noninteractive
-                      buffer-read-only
-                      (buffer-base-buffer) ;; Do not enable in indirect buffers
-                      (eq (aref (buffer-name) 0) ?\s)))
-             ;; TODO use `:predicate' on Emacs 29
-             (or (eq t global-jinx-modes)
-                 (eq t (cl-loop for p in global-jinx-modes thereis
-                                (pcase-exhaustive p
-                                  ('t t)
-                                  ('nil 0)
-                                  ((pred symbolp) (and (derived-mode-p p) t))
-                                  (`(not . ,m) (and (seq-some #'derived-mode-p m) 0)))))))
+  (unless (or noninteractive
+              buffer-read-only
+              (buffer-base-buffer) ;; Do not enable in indirect buffers
+              (eq (aref (buffer-name) 0) ?\s))
     (jinx-mode)))
 
 (provide 'jinx)
